<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ApprovedJobsAction as ApprovedJobsActionEvent;
use Assetise\Asset\Notifications\ApprovedJobsAction as ApprovedJobsActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ApprovedJobsAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ApprovedJobsActionEvent.
     *
     * @param   ApprovedJobsActionEvent  $event
     * @return mixed
     */
    public function handle(ApprovedJobsActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $approved_jobs.
     *
     * @param   ApprovedJobsActionEvent  $event
     * @return void
     */
    public function create(ApprovedJobsActionEvent $event)
    {
        $client = $event->$approved_jobs->client;
        Notification::send($client, new ApprovedJobsActionNotification($event));
    }

    /**
     * Handle the ApprovedJobsActionEvent as a listener.
     *
     * @param   ApprovedJobsActionEvent  $event
     * @return mixed
     */
    public function asListener(ApprovedJobsActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ApprovedJobsActionEvent  $event
     * @return bool
     */
    private function isAllowed(ApprovedJobsActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
