<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ApprovedJobsWorkflow as ApprovedJobsWorkflowEvent;
use Assetise\Asset\Notifications\ApprovedJobsWorkflow as ApprovedJobsWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ApprovedJobsWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ApprovedJobsWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $approved_jobs is submitted.
     *
     * @param ApprovedJobsWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ApprovedJobsWorkflowEvent $event)
    {
        $client = $event->$approved_jobs->client;
        Notification::send($client, new ApprovedJobsWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $approved_jobs is published.
     *
     * @param ApprovedJobsWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ApprovedJobsWorkflowEvent $event)
    {
        $client = $event->$approved_jobs->client;
        Notification::send($client, new ApprovedJobsWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $approved_jobs is approved.
     *
     * @param ApprovedJobsWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ApprovedJobsWorkflowEvent $event)
    {
        $client = $event->$approved_jobs->client;
        Notification::send($client, new ApprovedJobsWorkflowNotification($event));
    }

    /**
     * Handles the $approved_jobs workflow event as a listener.
     *
     * @param ApprovedJobsWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ApprovedJobsWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
