<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetAction as AssetActionEvent;
use Assetise\Asset\Notifications\AssetAction as AssetActionNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the AssetActionEvent.
     *
     * @param   AssetActionEvent  $event
     * @return mixed
     */
    public function handle(AssetActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $asset.
     *
     * @param   AssetActionEvent  $event
     * @return void
     */
    public function create(AssetActionEvent $event)
    {
        $client = $event->$asset->client;
        Notification::send($client, new AssetActionNotification($event));
    }

    /**
     * Handle the AssetActionEvent as a listener.
     *
     * @param   AssetActionEvent  $event
     * @return mixed
     */
    public function asListener(AssetActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   AssetActionEvent  $event
     * @return bool
     */
    private function isAllowed(AssetActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
