<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetClassificationAction as AssetClassificationActionEvent;
use Assetise\Asset\Notifications\AssetClassificationAction as AssetClassificationActionNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetClassificationAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the AssetClassificationActionEvent.
     *
     * @param   AssetClassificationActionEvent  $event
     * @return mixed
     */
    public function handle(AssetClassificationActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $asset_classification.
     *
     * @param   AssetClassificationActionEvent  $event
     * @return void
     */
    public function create(AssetClassificationActionEvent $event)
    {
        $client = $event->$asset_classification->client;
        Notification::send($client, new AssetClassificationActionNotification($event));
    }

    /**
     * Handle the AssetClassificationActionEvent as a listener.
     *
     * @param   AssetClassificationActionEvent  $event
     * @return mixed
     */
    public function asListener(AssetClassificationActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   AssetClassificationActionEvent  $event
     * @return bool
     */
    private function isAllowed(AssetClassificationActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
