<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetClassificationWorkflow as AssetClassificationWorkflowEvent;
use Assetise\Asset\Notifications\AssetClassificationWorkflow as AssetClassificationWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetClassificationWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(AssetClassificationWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $asset_classification is submitted.
     *
     * @param AssetClassificationWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(AssetClassificationWorkflowEvent $event)
    {
        $client = $event->$asset_classification->client;
        Notification::send($client, new AssetClassificationWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_classification is published.
     *
     * @param AssetClassificationWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(AssetClassificationWorkflowEvent $event)
    {
        $client = $event->$asset_classification->client;
        Notification::send($client, new AssetClassificationWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_classification is approved.
     *
     * @param AssetClassificationWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(AssetClassificationWorkflowEvent $event)
    {
        $client = $event->$asset_classification->client;
        Notification::send($client, new AssetClassificationWorkflowNotification($event));
    }

    /**
     * Handles the $asset_classification workflow event as a listener.
     *
     * @param AssetClassificationWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(AssetClassificationWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
