<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetLevelWorkflow as AssetLevelWorkflowEvent;
use Assetise\Asset\Notifications\AssetLevelWorkflow as AssetLevelWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetLevelWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(AssetLevelWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $asset_level is submitted.
     *
     * @param AssetLevelWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(AssetLevelWorkflowEvent $event)
    {
        $client = $event->$asset_level->client;
        Notification::send($client, new AssetLevelWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_level is published.
     *
     * @param AssetLevelWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(AssetLevelWorkflowEvent $event)
    {
        $client = $event->$asset_level->client;
        Notification::send($client, new AssetLevelWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_level is approved.
     *
     * @param AssetLevelWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(AssetLevelWorkflowEvent $event)
    {
        $client = $event->$asset_level->client;
        Notification::send($client, new AssetLevelWorkflowNotification($event));
    }

    /**
     * Handles the $asset_level workflow event as a listener.
     *
     * @param AssetLevelWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(AssetLevelWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
