<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetLocationAction as AssetLocationActionEvent;
use Assetise\Asset\Notifications\AssetLocationAction as AssetLocationActionNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetLocationAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the AssetLocationActionEvent.
     *
     * @param   AssetLocationActionEvent  $event
     * @return mixed
     */
    public function handle(AssetLocationActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $asset_location.
     *
     * @param   AssetLocationActionEvent  $event
     * @return void
     */
    public function create(AssetLocationActionEvent $event)
    {
        $client = $event->$asset_location->client;
        Notification::send($client, new AssetLocationActionNotification($event));
    }

    /**
     * Handle the AssetLocationActionEvent as a listener.
     *
     * @param   AssetLocationActionEvent  $event
     * @return mixed
     */
    public function asListener(AssetLocationActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   AssetLocationActionEvent  $event
     * @return bool
     */
    private function isAllowed(AssetLocationActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
