<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetTypeAction as AssetTypeActionEvent;
use Assetise\Asset\Notifications\AssetTypeAction as AssetTypeActionNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetTypeAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the AssetTypeActionEvent.
     *
     * @param   AssetTypeActionEvent  $event
     * @return mixed
     */
    public function handle(AssetTypeActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $asset_type.
     *
     * @param   AssetTypeActionEvent  $event
     * @return void
     */
    public function create(AssetTypeActionEvent $event)
    {
        $client = $event->$asset_type->client;
        Notification::send($client, new AssetTypeActionNotification($event));
    }

    /**
     * Handle the AssetTypeActionEvent as a listener.
     *
     * @param   AssetTypeActionEvent  $event
     * @return mixed
     */
    public function asListener(AssetTypeActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   AssetTypeActionEvent  $event
     * @return bool
     */
    private function isAllowed(AssetTypeActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
