<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetTypeWorkflow as AssetTypeWorkflowEvent;
use Assetise\Asset\Notifications\AssetTypeWorkflow as AssetTypeWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetTypeWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(AssetTypeWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $asset_type is submitted.
     *
     * @param AssetTypeWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(AssetTypeWorkflowEvent $event)
    {
        $client = $event->$asset_type->client;
        Notification::send($client, new AssetTypeWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_type is published.
     *
     * @param AssetTypeWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(AssetTypeWorkflowEvent $event)
    {
        $client = $event->$asset_type->client;
        Notification::send($client, new AssetTypeWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset_type is approved.
     *
     * @param AssetTypeWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(AssetTypeWorkflowEvent $event)
    {
        $client = $event->$asset_type->client;
        Notification::send($client, new AssetTypeWorkflowNotification($event));
    }

    /**
     * Handles the $asset_type workflow event as a listener.
     *
     * @param AssetTypeWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(AssetTypeWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
