<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\AssetWorkflow as AssetWorkflowEvent;
use Assetise\Asset\Notifications\AssetWorkflow as AssetWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class AssetWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(AssetWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $asset is submitted.
     *
     * @param AssetWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(AssetWorkflowEvent $event)
    {
        $client = $event->$asset->client;
        Notification::send($client, new AssetWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset is published.
     *
     * @param AssetWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(AssetWorkflowEvent $event)
    {
        $client = $event->$asset->client;
        Notification::send($client, new AssetWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $asset is approved.
     *
     * @param AssetWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(AssetWorkflowEvent $event)
    {
        $client = $event->$asset->client;
        Notification::send($client, new AssetWorkflowNotification($event));
    }

    /**
     * Handles the $asset workflow event as a listener.
     *
     * @param AssetWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(AssetWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
