<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ConsequenceAction as ConsequenceActionEvent;
use Assetise\Asset\Notifications\ConsequenceAction as ConsequenceActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ConsequenceAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ConsequenceActionEvent.
     *
     * @param   ConsequenceActionEvent  $event
     * @return mixed
     */
    public function handle(ConsequenceActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $consequence.
     *
     * @param   ConsequenceActionEvent  $event
     * @return void
     */
    public function create(ConsequenceActionEvent $event)
    {
        $client = $event->$consequence->client;
        Notification::send($client, new ConsequenceActionNotification($event));
    }

    /**
     * Handle the ConsequenceActionEvent as a listener.
     *
     * @param   ConsequenceActionEvent  $event
     * @return mixed
     */
    public function asListener(ConsequenceActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ConsequenceActionEvent  $event
     * @return bool
     */
    private function isAllowed(ConsequenceActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
