<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ConsequenceWorkflow as ConsequenceWorkflowEvent;
use Assetise\Asset\Notifications\ConsequenceWorkflow as ConsequenceWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ConsequenceWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ConsequenceWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $consequence is submitted.
     *
     * @param ConsequenceWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ConsequenceWorkflowEvent $event)
    {
        $client = $event->$consequence->client;
        Notification::send($client, new ConsequenceWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $consequence is published.
     *
     * @param ConsequenceWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ConsequenceWorkflowEvent $event)
    {
        $client = $event->$consequence->client;
        Notification::send($client, new ConsequenceWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $consequence is approved.
     *
     * @param ConsequenceWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ConsequenceWorkflowEvent $event)
    {
        $client = $event->$consequence->client;
        Notification::send($client, new ConsequenceWorkflowNotification($event));
    }

    /**
     * Handles the $consequence workflow event as a listener.
     *
     * @param ConsequenceWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ConsequenceWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
