<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ExpenditureAction as ExpenditureActionEvent;
use Assetise\Asset\Notifications\ExpenditureAction as ExpenditureActionNotification;
use Litepie\Actions\Concerns\AsAction;

class ExpenditureAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the ExpenditureActionEvent.
     *
     * @param   ExpenditureActionEvent  $event
     * @return mixed
     */
    public function handle(ExpenditureActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $expenditure.
     *
     * @param   ExpenditureActionEvent  $event
     * @return void
     */
    public function create(ExpenditureActionEvent $event)
    {
        $client = $event->$expenditure->client;
        Notification::send($client, new ExpenditureActionNotification($event));
    }

    /**
     * Handle the ExpenditureActionEvent as a listener.
     *
     * @param   ExpenditureActionEvent  $event
     * @return mixed
     */
    public function asListener(ExpenditureActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   ExpenditureActionEvent  $event
     * @return bool
     */
    private function isAllowed(ExpenditureActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
