<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\ExpenditureWorkflow as ExpenditureWorkflowEvent;
use Assetise\Asset\Notifications\ExpenditureWorkflow as ExpenditureWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class ExpenditureWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(ExpenditureWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $expenditure is submitted.
     *
     * @param ExpenditureWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(ExpenditureWorkflowEvent $event)
    {
        $client = $event->$expenditure->client;
        Notification::send($client, new ExpenditureWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $expenditure is published.
     *
     * @param ExpenditureWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(ExpenditureWorkflowEvent $event)
    {
        $client = $event->$expenditure->client;
        Notification::send($client, new ExpenditureWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $expenditure is approved.
     *
     * @param ExpenditureWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(ExpenditureWorkflowEvent $event)
    {
        $client = $event->$expenditure->client;
        Notification::send($client, new ExpenditureWorkflowNotification($event));
    }

    /**
     * Handles the $expenditure workflow event as a listener.
     *
     * @param ExpenditureWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(ExpenditureWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
