<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\InspectionWorkflow as InspectionWorkflowEvent;
use Assetise\Asset\Notifications\InspectionWorkflow as InspectionWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class InspectionWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(InspectionWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $inspection is submitted.
     *
     * @param InspectionWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(InspectionWorkflowEvent $event)
    {
        $client = $event->$inspection->client;
        Notification::send($client, new InspectionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $inspection is published.
     *
     * @param InspectionWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(InspectionWorkflowEvent $event)
    {
        $client = $event->$inspection->client;
        Notification::send($client, new InspectionWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $inspection is approved.
     *
     * @param InspectionWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(InspectionWorkflowEvent $event)
    {
        $client = $event->$inspection->client;
        Notification::send($client, new InspectionWorkflowNotification($event));
    }

    /**
     * Handles the $inspection workflow event as a listener.
     *
     * @param InspectionWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(InspectionWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
