<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\LccAction as LccActionEvent;
use Assetise\Asset\Notifications\LccAction as LccActionNotification;
use Litepie\Actions\Concerns\AsAction;

class LccAction
{
    use AsAction;

    private $allowedActions = [
        'before' => [],
        'after' => ['create'],
    ];

    /**
     * Handle the LccActionEvent.
     *
     * @param   LccActionEvent  $event
     * @return mixed
     */
    public function handle(LccActionEvent $event)
    {
        $function = Str::camel($event->action);
        return $this->$function($event);
    }

    /**
     * Create a new $lcc.
     *
     * @param   LccActionEvent  $event
     * @return void
     */
    public function create(LccActionEvent $event)
    {
        $client = $event->$lcc->client;
        Notification::send($client, new LccActionNotification($event));
    }

    /**
     * Handle the LccActionEvent as a listener.
     *
     * @param   LccActionEvent  $event
     * @return mixed
     */
    public function asListener(LccActionEvent $event)
    {
        if ($this->isAllowed($event)) {
            return $this->handle($event);
        }
    }

    /**
     * Check if the event action is allowed.
     *
     * @param   LccActionEvent  $event
     * @return bool
     */
    private function isAllowed(LccActionEvent $event)
    {
        if ($event->when == 'before' &&
            !in_array($event->action, $this->allowedActions['before'])) {
            return false;
        }

        if (($event->when == 'after' &&
            !in_array($event->action, $this->allowedActions['after']))
        ) {
            return false;
        }

        return true;
    }
}
