<?php

namespace Assetise\Asset\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Assetise\Asset\Events\LccWorkflow as LccWorkflowEvent;
use Assetise\Asset\Notifications\LccWorkflow as LccWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class LccWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(LccWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $lcc is submitted.
     *
     * @param LccWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(LccWorkflowEvent $event)
    {
        $client = $event->$lcc->client;
        Notification::send($client, new LccWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $lcc is published.
     *
     * @param LccWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(LccWorkflowEvent $event)
    {
        $client = $event->$lcc->client;
        Notification::send($client, new LccWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $lcc is approved.
     *
     * @param LccWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(LccWorkflowEvent $event)
    {
        $client = $event->$lcc->client;
        Notification::send($client, new LccWorkflowNotification($event));
    }

    /**
     * Handles the $lcc workflow event as a listener.
     *
     * @param LccWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(LccWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
